@@ -2,13 +2,11 @@
 # BroControl Plugin API.
 #
 
-import pluginreg
-import config
-import util
-import doc
-import execute
+from __future__ import print_function
+import logging
 
-Registry = pluginreg.PluginRegistry()
+from BroControl import config
+from BroControl import doc
 
 class Plugin(object):
     """The class ``Plugin`` is the base class for all BroControl plugins.
@@ -66,8 +64,8 @@ class Plugin(object):
         attribute *name*. If the user has not set the options, its default
         value is returned. See the output of ``broctl config`` for a complete
         list."""
-        if not config.Config.hasAttr(name):
-            raise KeyError
+        if not config.Config.has_attr(name):
+            raise KeyError("unknown config option %s" % name)
 
         return config.Config.__getattr__(name)
 
@@ -82,8 +80,8 @@ class Plugin(object):
         """
         name = "%s.%s" % (self.prefix().lower(), name.lower())
 
-        if not config.Config.hasAttr(name):
-            raise KeyError
+        if not config.Config.has_attr(name):
+            raise KeyError("unknown plugin option %s" % name)
 
         return config.Config.__getattr__(name)
 
@@ -100,7 +98,7 @@ class Plugin(object):
         """
         name = "%s.state.%s" % (self.prefix().lower(), name.lower())
 
-        if not config.Config.hasAttr(name):
+        if not config.Config.has_attr(name):
             return ""
 
         return config.Config.__getattr__(name)
@@ -121,47 +119,52 @@ class Plugin(object):
             self.error("plugin state variable names must not contain dots or spaces")
 
         name = "%s.state.%s" % (self.prefix(), name)
-        config.Config._setState(name, value)
+        config.Config.set_state(name, value)
 
     @doc.api
     def parseNodes(self, names):
-        """Returns a list of `Node`_ objects for a string of space-separated
-        node names. If a name does not correspond to a known node, an error
-        message is printed and the node is skipped from the returned list. If
-        no names are known, an empty list is returned."""
+        """Returns a tuple which contains two lists. The first list is a list
+        of `Node`_ objects for a string of space-separated node names. If a
+        name does not correspond to a known node, then the name is added
+        to the second list in the returned tuple.
+        """
         nodes = []
+        notnodes = []
 
         for arg in names.split():
-            h = config.Config.nodes(arg, True)
-            if not h:
-                util.output("unknown node '%s'" % arg)
+            nodelist = config.Config.nodes(arg)
+            if nodelist:
+                nodes += nodelist
             else:
-                nodes += h
+                notnodes.append(arg)
 
-        return nodes
+        # Sort the list so that it doesn't depend on initial order of arguments
+        nodes.sort(key=lambda n: (n.type, n.name))
+
+        return (nodes, notnodes)
 
     @doc.api
     def message(self, msg):
         """Reports a message to the user."""
-        util.output(msg, prefix="plugin:%s" % self.prefix())
+        print("%s" % msg)
 
     @doc.api
     def debug(self, msg):
         """Logs a debug message in BroControl's debug log if enabled."""
-        util.debug(1, msg, prefix="plugin:%s" % self.prefix())
+        logging.debug("%s: %s", self.prefix(), msg)
 
     @doc.api
     def error(self, msg):
-        """Reports an error to the user and terminates broctl."""
-        util.error(msg, prefix="plugin:%s" % self.prefix())
+        """Reports an error to the user."""
+        print("error: %s" % msg)
 
     @doc.api
     def execute(self, node, cmd):
         """Executes a command on the host for the given *node* of type
         `Node`_. Returns a tuple ``(success, output)`` in which ``success`` is
         True if the command ran successfully and ``output`` is the combined
-        stdout/stderr output (or None if we couldn't connect to the host)."""
-        result = execute.executeCmdsParallel([(node, cmd)])[0]
+        stdout/stderr output."""
+        result = self.executor.run_shell_cmds([(node, cmd)])[0]
         return (result[1], result[2])
 
     @doc.api
@@ -196,9 +199,8 @@ class Plugin(object):
         instance and *cmd* is a string with the command to execute for it. The
         method returns a list of tuples ``(node, success, output)``, in which
         ``success`` is True if the command ran successfully and ``output`` is
-        the combined stdout/stderr output (or None if we couldn't connect to
-        the host) for the corresponding ``node``."""
-        return execute.executeCmdsParallel(cmds)
+        the combined stdout/stderr output for the corresponding ``node``."""
+        return self.executor.run_shell_cmds(cmds)
 
     ### Methods that must be overridden by plugins.
 
@@ -558,6 +560,25 @@ class Plugin(object):
         """
         pass
 
+    @doc.api("override")
+    def cmd_deploy_pre(self):
+        """Called just before the ``deploy`` command is run. Returns a
+        boolean indicating whether or not the command should run.
+
+        This method can be overridden by derived classes. The default
+        implementation does nothing.
+        """
+        return True
+
+    @doc.api("override")
+    def cmd_deploy_post(self):
+        """Called just after the ``deploy`` command has finished.
+
+        This method can be overridden by derived classes. The default
+        implementation does nothing.
+        """
+        pass
+
     @doc.api("override")
     def cmd_status_pre(self, nodes):
         """Called just before the ``status`` command is run. It receives the
@@ -602,7 +623,7 @@ class Plugin(object):
         pass
 
     @doc.api("override")
-    def cmd_custom(self, cmd, args):
+    def cmd_custom(self, cmd, args, cmdout):
         """Called when a command defined by the ``commands`` method is executed.
         *cmd* is the command (without the plugin's prefix), and *args* is a
         single string with all arguments.
@@ -657,27 +678,6 @@ class Plugin(object):
         """
         pass
 
-    @doc.api("override")
-    def cmd_attachgdb_pre(self, nodes):
-        """Called just before the ``attachgdb`` command is run. It receives the
-        list of nodes, and returns the list of nodes that should proceed with
-        the command.
-
-        This method can be overridden by derived classes. The default
-        implementation does nothing.
-        """
-        pass
-
-    @doc.api("override")
-    def cmd_attachgdb_post(self, nodes):
-        """Called just after the ``attachgdb`` command has finished. Arguments
-        are as with the ``pre`` method.
-
-        This method can be overridden by derived classes. The default
-        implementation does nothing.
-        """
-        pass
-
     @doc.api("override")
     def cmd_peerstatus_pre(self, nodes):
         """Called just before the ``peerstatus`` command is run. It receives the
@@ -898,4 +898,4 @@ class Plugin(object):
             if not isinstance(default, str):
                 self.error("plugin option default must be a string")
 
-            config.Config._setOption("%s.%s" % (self.prefix(), name), default)
+            config.Config._set_option("%s.%s" % (self.prefix(), name), default)
